package nu.helmers.commons.guice.test;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.easymock.EasyMock;

import com.google.inject.Guice;
import com.google.inject.Injector;
import com.google.inject.Key;
import com.google.inject.Module;
import com.google.inject.util.Modules;

/**
 * Wrapper for a Guice {@link Injector} that mocks specific interfaces. Note that you can even mock interfaces that are
 * not normally configured by Guice.
 */
public class MockInjector {

    protected final Injector injector;

    private final List<Key<?>> mockedKeys = new ArrayList<Key<?>>();

    public MockInjector(List<MockModule> mockedModules) {
        addAllMockedKeys(mockedModules);
        injector = Guice.createInjector(mockedModules);
    }

    public MockInjector(Module module, MockModule mockModule) {
        addAllMockedKeys(Arrays.<MockModule> asList(mockModule));
        injector = Guice.createInjector(Modules.override(module).with(new MockModule(this.mockedKeys)));
    }

    @SuppressWarnings("unused")
    private MockInjector(MockModule... mockedModules) {
        throw new RuntimeException(
                "Don't use this constructor! It shadows the (Module, MockModule) constructor, leading to nasty bugs!");
    }

    private void addAllMockedKeys(List<MockModule> modules) {
        for (MockModule module : modules) {
            mockedKeys.addAll(module.getMockedKeys());
        }
    }

    /** retrieve object from the underlying injector */
    public <T> T get(Class<T> clazz) {
        return injector.getInstance(clazz);
    }

    /** retrieve object from underlying injector */
    public <T> T get(Key<T> key) {
        return injector.getInstance(key);
    }

    /** put all mock objects in replay mode */
    public void replay() {
        for (Key<?> key : mockedKeys) {
            EasyMock.replay(get(key));
        }
    }

    /** verify all mock objects have seen the expected invocations */
    public void verify() {
        for (Key<?> key : mockedKeys) {
            EasyMock.verify(get(key));
        }
    }

    public void injectMembers(Object object) {
        injector.injectMembers(object);
    }
}
