package nu.helmers.commons.guice.test;

import static org.easymock.EasyMock.createMock;

import java.util.List;

import com.google.inject.AbstractModule;
import com.google.inject.Key;
import com.google.inject.Module;
import com.google.inject.binder.AnnotatedBindingBuilder;

/**
 * A Guice {@link Module} that provides mocks for keys indicated in the constructor. This class should is used in
 * combination with {@link MockInjector}
 */
public class MockModule extends AbstractModule {
    private final List<Key<?>> mockedKeys;

    public MockModule(List<Key<?>> mockedKeys) {
        super();
        this.mockedKeys = mockedKeys;
    }

    @Override
    @SuppressWarnings("unchecked")
    protected void configure() {
        for (Key<?> key : mockedKeys) {
            ((AnnotatedBindingBuilder<Object>) bind(key)).toInstance(createMock(interfaceToMock(key)));
        }
    }

    /** the interface to be mocked for this key */
    public Class<?> interfaceToMock(Key<?> key) {
        return key.getTypeLiteral().getRawType();
    }

    public List<Key<?>> getMockedKeys() {
        return mockedKeys;
    }
}
