package nu.helmers.jbpmtest.hibernate;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.CriteriaSpecification;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Projection;
import org.hibernate.criterion.Projections;
import org.springframework.beans.factory.InitializingBean;

public abstract class AbstractGenericDao<T> implements InitializingBean, IGenericDao<T> {

    private GenericSessionFactory sessionFactory;

    private Class<T> entityBeanType;

    protected AbstractGenericDao(SessionFactory sessionFactory, Class<T> clazz) {
        setSessionFactory(sessionFactory);
        this.entityBeanType = clazz;
    }

    protected AbstractGenericDao(Class<T> clazz) {
        this.entityBeanType = clazz;
    }

    public void afterPropertiesSet() throws Exception {
        if (sessionFactory == null) {
            throw new IllegalStateException("Required property sessionFactory was not set");
        }
    }

    protected void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = new GenericSessionFactory(sessionFactory);
    }

    protected GenericSession session() {
        return sessionFactory.getCurrentSession();
    }

    public Class<T> getEntityBeanType() {
        return entityBeanType;
    }

    public T get(Long id) {
        T object = session().get(entityBeanType, id);
        if (object == null) {
            throw new RuntimeException("No entity found for id " + id);
        }
        return object;
    }

    public int count() {
        return getUnique(Projections.count("id"));
    }

    public void persist(T entity) {
        session().persist(entity);
    }

    public void update(T entity) {
        session().update(entity);
    }

    public void delete(T entity) {
        session().delete(entity);
    }

    @SuppressWarnings("unchecked")
    protected T getUnique(Criterion... criterions) {
        Criteria criteria = criteria();
        for (Criterion criterion : criterions) {
            criteria.add(criterion);
        }
        return (T) criteria.uniqueResult();
    }

    protected int getUnique(Projection projection, Criterion... criterions) {
        Criteria criteria = criteria();
        for (Criterion criterion : criterions) {
            criteria.add(criterion);
        }
        criteria.setProjection(projection);
        return ((Number) criteria.uniqueResult()).intValue();
    }

    @SuppressWarnings("unchecked")
    public List<T> findAll() {
        return criteria().setResultTransformer(CriteriaSpecification.DISTINCT_ROOT_ENTITY).list();
    }

    protected Criteria criteria() {
        return session().createCriteria(entityBeanType);
    }

}
