package nu.helmers.jbpmtest.hibernate;

import java.io.Serializable;
import java.sql.Connection;

import org.hibernate.CacheMode;
import org.hibernate.Criteria;
import org.hibernate.EntityMode;
import org.hibernate.Filter;
import org.hibernate.FlushMode;
import org.hibernate.HibernateException;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.ReplicationMode;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.stat.SessionStatistics;

public class GenericSession {
    private final Session session;

    public GenericSession(Session session) {
        this.session = session;
    }

    public Transaction beginTransaction() throws HibernateException {
        return session.beginTransaction();
    }

    public void cancelQuery() throws HibernateException {
        session.cancelQuery();
    }

    public void clear() {
        session.clear();
    }

    public Connection close() throws HibernateException {
        return session.close();
    }

    @Deprecated
    public Connection connection() throws HibernateException {
        return session.connection();
    }

    public boolean contains(Object object) {
        return session.contains(object);
    }

    public Criteria createCriteria(Class<?> persistentClass, String alias) {
        return session.createCriteria(persistentClass, alias);
    }

    public Criteria createCriteria(Class<?> persistentClass) {
        return session.createCriteria(persistentClass);
    }

    public Criteria createCriteria(String entityName, String alias) {
        return session.createCriteria(entityName, alias);
    }

    public Criteria createCriteria(String entityName) {
        return session.createCriteria(entityName);
    }

    public SQLQuery createSQLQuery(String queryString) throws HibernateException {
        return session.createSQLQuery(queryString);
    }

    public void delete(Object object) throws HibernateException {
        session.delete(object);
    }

    public void delete(String entityName, Object object) throws HibernateException {
        session.delete(entityName, object);
    }

    public void disableFilter(String filterName) {
        session.disableFilter(filterName);
    }

    public Connection disconnect() throws HibernateException {
        return session.disconnect();
    }

    public Filter enableFilter(String filterName) {
        return session.enableFilter(filterName);
    }

    public void evict(Object object) throws HibernateException {
        session.evict(object);
    }

    public void flush() throws HibernateException {
        session.flush();
    }

    public <T> T get(Class<T> clazz, Serializable id, LockMode lockMode) throws HibernateException {
        return clazz.cast(session.get(clazz, id, lockMode));
    }

    public <T> T get(Class<T> clazz, Serializable id) throws HibernateException {
        return clazz.cast(session.get(clazz, id));
    }

    @SuppressWarnings("unchecked")
    public <T> T get(String entityName, Serializable id, LockMode lockMode) throws HibernateException {
        return (T) session.get(entityName, id, lockMode);
    }

    @SuppressWarnings("unchecked")
    public <T> T get(String entityName, Serializable id) throws HibernateException {
        return (T) session.get(entityName, id);
    }

    public CacheMode getCacheMode() {
        return session.getCacheMode();
    }

    public LockMode getCurrentLockMode(Object object) throws HibernateException {
        return session.getCurrentLockMode(object);
    }

    public Filter getEnabledFilter(String filterName) {
        return session.getEnabledFilter(filterName);
    }

    public EntityMode getEntityMode() {
        return session.getEntityMode();
    }

    public String getEntityName(Object object) throws HibernateException {
        return session.getEntityName(object);
    }

    public FlushMode getFlushMode() {
        return session.getFlushMode();
    }

    public Serializable getIdentifier(Object object) throws HibernateException {
        return session.getIdentifier(object);
    }

    public Query getNamedQuery(String queryName) throws HibernateException {
        return session.getNamedQuery(queryName);
    }

    public Session getSession(EntityMode entityMode) {
        return session.getSession(entityMode);
    }

    public SessionFactory getSessionFactory() {
        return session.getSessionFactory();
    }

    public SessionStatistics getStatistics() {
        return session.getStatistics();
    }

    public Transaction getTransaction() {
        return session.getTransaction();
    }

    public boolean isConnected() {
        return session.isConnected();
    }

    public boolean isDirty() throws HibernateException {
        return session.isDirty();
    }

    public boolean isOpen() {
        return session.isOpen();
    }

    public <T> T load(Class<T> clazz, Serializable id, LockMode lockMode) throws HibernateException {
        return clazz.cast(session.load(clazz, id, lockMode));
    }

    public <T> T load(Class<T> clazz, Serializable id) throws HibernateException {
        return clazz.cast(session.load(clazz, id));
    }

    public void load(Object object, Serializable id) throws HibernateException {
        session.load(object, id);
    }

    @SuppressWarnings("unchecked")
    public <T> T load(String entityName, Serializable id, LockMode lockMode) throws HibernateException {
        return (T) session.load(entityName, id, lockMode);
    }

    @SuppressWarnings("unchecked")
    public <T> T load(String entityName, Serializable id) throws HibernateException {
        return (T) session.load(entityName, id);
    }

    public void lock(Object object, LockMode lockMode) throws HibernateException {
        session.lock(object, lockMode);
    }

    public void lock(String entityName, Object object, LockMode lockMode) throws HibernateException {
        session.lock(entityName, object, lockMode);
    }

    @SuppressWarnings("unchecked")
    public <T> T merge(T object) throws HibernateException {
        return (T) session.merge(object);
    }

    @SuppressWarnings("unchecked")
    public <T> T merge(String entityName, T object) throws HibernateException {
        return (T) session.merge(entityName, object);
    }

    public void persist(Object object) throws HibernateException {
        session.persist(object);
    }

    public void persist(String entityName, Object object) throws HibernateException {
        session.persist(entityName, object);
    }

    @Deprecated
    public void reconnect() throws HibernateException {
        session.reconnect();
    }

    public void reconnect(Connection connection) throws HibernateException {
        session.reconnect(connection);
    }

    public void refresh(Object object, LockMode lockMode) throws HibernateException {
        session.refresh(object, lockMode);
    }

    public void refresh(Object object) throws HibernateException {
        session.refresh(object);
    }

    public void replicate(Object object, ReplicationMode replicationMode) throws HibernateException {
        session.replicate(object, replicationMode);
    }

    public void replicate(String entityName, Object object, ReplicationMode replicationMode)
            throws HibernateException {
        session.replicate(entityName, object, replicationMode);
    }

    public Serializable save(Object object) throws HibernateException {
        return session.save(object);
    }

    public Serializable save(String entityName, Object object) throws HibernateException {
        return session.save(entityName, object);
    }

    public void saveOrUpdate(Object object) throws HibernateException {
        session.saveOrUpdate(object);
    }

    public void saveOrUpdate(String entityName, Object object) throws HibernateException {
        session.saveOrUpdate(entityName, object);
    }

    public void setCacheMode(CacheMode cacheMode) {
        session.setCacheMode(cacheMode);
    }

    public void setFlushMode(FlushMode flushMode) {
        session.setFlushMode(flushMode);
    }

    public void setReadOnly(Object entity, boolean readOnly) {
        session.setReadOnly(entity, readOnly);
    }

    public void update(Object object) throws HibernateException {
        session.update(object);
    }

    public void update(String entityName, Object object) throws HibernateException {
        session.update(entityName, object);
    }

}
