package nu.helmers.jbpmtest.hibernate;

import java.util.List;

/**
 * @param <T>
 *            the type of entity this dao operates on.
 */
public interface IGenericDao<T> {

    /**
     * @return all entities of type {@link T}.
     */
    List<T> findAll();

    /**
     * @return the entity with the given <code>id</code>. RuntimeException if no entity of the given type exists for the
     *         specified id.
     */
    T get(Long primaryKey);

    /**
     * @param entity
     *            the entity to persist
     */
    void persist(T entity);

    /**
     * @param entity
     *            the entity to update
     */
    void update(T entity);

    /**
     * @param entity
     *            the entity to delete
     */
    void delete(T entity);

    /**
     * @return the total number of entities of the specified type
     */
    int count();

}
