package nu.helmers.jbpmtest.wiring.modules;

import com.google.inject.AbstractModule;
import com.google.inject.Module;
import com.google.inject.matcher.Matcher;
import com.wideplay.warp.persist.PersistenceModuleBuilder;
import com.wideplay.warp.persist.PersistenceService;
import com.wideplay.warp.persist.TransactionStrategyBuilder;
import com.wideplay.warp.persist.UnitOfWork;

/** Wrapper around Warp's {@link PersistenceModuleBuilder}, to simplify construction of 'custom' transactional behavior. */
public class PersistenceModule extends AbstractModule implements Module {

    @SuppressWarnings("unchecked")
    private Matcher<Class> transactionalClassMatcher;

    private Matcher<Object> transactionalMethodMatcher;

    /** Creates a module in which every @Transactional method is transactional, and nothing else. */
    public PersistenceModule() {
        super();
    }

    /**
     * Use this constructor to specify which classes and/or methods, that are not annotated with @Transactional, should
     * be wrapped in a transaction by warp (in addition to @Transactional methods which will always be transactional).
     * Typical use case is integration testing dao methods (which are not @Transactional).
     * 
     * @param classes
     *            classes that should be transactional
     * @param methods
     *            methods in the specified classes that should be transactional
     */
    @SuppressWarnings("unchecked")
    public PersistenceModule(Matcher<Class> classes, Matcher<Object> methods) {
        super();
        if (classes == null || methods == null) {
            throw new IllegalArgumentException("All arguments of this constructor are required.");
        }
        this.transactionalClassMatcher = classes;
        this.transactionalMethodMatcher = methods;
    }

    @Override
    protected void configure() {
        TransactionStrategyBuilder builder = PersistenceService.usingHibernate().across(UnitOfWork.TRANSACTION);
        if (this.transactionalClassMatcher != null) {
            builder = builder.forAll(transactionalClassMatcher, transactionalMethodMatcher);
            // Does this *override* the @Transactional methods, or is it an addition?
            // (in other words will @Transactional methods still be transactional after this .forAll()).
        }
        install(builder.buildModule());
    }

}
