package nu.helmers.jbpmtest.wiring.providers;

import static nu.helmers.jbpmtest.DataLayerProperties.HIBERNATE_DIALECT;
import static nu.helmers.jbpmtest.DataLayerProperties.HIBERNATE_HBM2DDL_AUTO;
import nu.helmers.jbpmtest.wiring.modules.DataLayerModule;

import org.hibernate.cfg.Configuration;

import com.google.inject.Inject;
import com.google.inject.Provider;
import com.google.inject.name.Named;

/**
 * This provider knows how to construct a Hibernate Configuration. It reads all settings from the static
 * hibernate.cfg.xml file (part of this data layer jar), except for a few settings that differ between unit /
 * integration testing and real deployments.
 * <p/>
 * It assumes that the following properties are available as @Named bindings:
 * <ul>
 * <li>jbpmtest.hibernate.dialect
 * <li>jbpmtest.hibernate.hbm2ddl.auto
 * </ul>
 * See {@link DataLayerModule} how to make these properties available.
 */
public class HibernateConfigurationProvider implements Provider<Configuration> {

    private static final String HIBERNATE_CFG_XML = "hibernate.cfg.xml";

    private final String hibernateDialect;

    private final String hibernateHbm2ddlAuto;

    @Inject
    public HibernateConfigurationProvider(@Named(HIBERNATE_DIALECT) String hibernateDialect,
            @Named(HIBERNATE_HBM2DDL_AUTO) String hibernateHbm2ddlAuto) {
        this.hibernateDialect = hibernateDialect;
        this.hibernateHbm2ddlAuto = hibernateHbm2ddlAuto;
    }

    public Configuration get() {
        return new Configuration().configure(HIBERNATE_CFG_XML).setProperty("hibernate.dialect", hibernateDialect)
                .setProperty("hibernate.hbm2ddl.auto", hibernateHbm2ddlAuto);
    }

}
