package nu.helmers.jbpmtest.dao.impl.issue;

import static junit.framework.Assert.assertEquals;
import nu.helmers.jbpmtest.dao.impl.GenericDaoTest;
import nu.helmers.jbpmtest.dao.issue.IIssueDao;
import nu.helmers.jbpmtest.domain.issue.Issue;
import nu.helmers.jbpmtest.fixtures.FixturesIssue;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.google.inject.Inject;

/**
 * Test the {@link IssueDaoImpl} against an in-memory database.
 */
public class IssueDaoIntegrationTest extends GenericDaoTest<Issue> {

    @Inject
    private IIssueDao issueDao;

    /** A issue object that can be persisted with issueDao, assumed that reference data are already persisted. */
    private Issue issue;

    @Before
    public void setup() {
        super.setup();
        issue = FixturesIssue.makeNewGbaIssue();
    }

    @After
    public void teardown() {
        try {
            super.teardown();
        } finally {
            issueDao = null;
        }
    }

    @Test
    public void testPersistAndGet() {
        testPersistAndGet(issueDao, issue);
    }

    @Test
    public void testGetThrowsExceptionForNonExistingId() {
        testGetThrowsExceptionForNonExistingId(issueDao);
    }

    @Test
    public void testFindAll() {
        testFindAll(issueDao, issue);
    }

    @Test
    public void testDelete() {
        testDelete(issueDao, issue);
    }

    @Test
    public void testCount() {
        testCount(issueDao, issue);
    }

    @Test
    public void testUpdate() {
        issueDao.persist(issue);

        // Update it.
        Issue issuePersisted = issueDao.get(issue.getId());
        issuePersisted.setOmschrijving("Nieuwe omschrijving");
        issueDao.update(issuePersisted);

        // Check that the update was persisted.
        Issue issueUpdated = issueDao.get(issue.getId());
        assertEquals("Nieuwe omschrijving", issueUpdated.getOmschrijving());
    }

}
