package nu.helmers.jbpmtest.domain;

import java.io.Serializable;

/**
 * Superclass voor domein objecten die met Hibernate gemapt worden.
 */
public abstract class DomainObject implements Serializable {

    private static final long serialVersionUID = 197283277199387436L;

    private Long id;

    private long version;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public long getVersion() {
        return version;
    }

    public void setVersion(long version) {
        this.version = version;
    }

    /**
     * N.B. met deze methode zijn alle subclasses van DomainObject gelijk aanelkaar. Omdat hibernate mbv cglib dynamisch
     * proxies genereert, werkt het niet als je zegt <code>o.getClass() != getClass()</code>, want ook al gaat het om
     * hetzelfde domainobject, zeg "Issue", dan nog kan er een van de objecten een dynamisch gegenereerde subclass van
     * Issue zijn. Enige goedwerkende oplossing is om in de subclass een override te doen:
     * 
     * <pre>
     * &#064;Override
     * public boolean equals(Object o) {
     *     if (o == null || !(o instanceof MijnSubclass)) {
     *         return false;
     *     }
     *     return super.equals(o);
     * }
     * </pre>
     */
    @Override
    public boolean equals(Object o) {
        if (o == null || !(o instanceof DomainObject)) {
            return false;
        } else if (this == o) {
            return true;
        } else if (!getClass().isInstance(o)) {
            return false;
        }
        DomainObject other = (DomainObject) o;
        Long myId = getId();
        Long otherId = other.getId();
        if (myId == null) {
            return false;
        } else {
            return myId.equals(otherId);
        }
    }

    @Override
    public int hashCode() {
        return id == null ? 5 : id.hashCode();
    }
}
