package nu.helmers.jbpmtest.logging;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Method annotation intended to instruct the injection framework that this method should be logged.
 * <p>
 * Note that methods annotated @Transactional @LogMe need not show up in the logging: the transaction wrapper may hit a
 * database connection error even before the logging interceptor triggers.
 * <p>
 * Note that this is intended for unplanned exceptions at the service layer. Exceptions in eg Wicket page constructors
 * will be caught and logged by Wicket without need for an explicit handler in our code.
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
public @interface LogMe {
}
