package nu.helmers.jbpmtest.service.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import nu.helmers.jbpmtest.domain.issue.Issue;
import nu.helmers.jbpmtest.service.JbpmFacade;

import org.apache.commons.collections.CollectionUtils;
import org.jbpm.api.ExecutionService;
import org.jbpm.api.ProcessDefinition;
import org.jbpm.api.ProcessDefinitionQuery;
import org.jbpm.api.ProcessInstance;
import org.jbpm.api.RepositoryService;
import org.jbpm.api.TaskService;
import org.jbpm.api.task.Task;

import com.google.inject.Inject;

public class JbpmFacadeImpl implements JbpmFacade {

    static final String VAR_ISSUE_ID = "issueId";

    private TaskService taskService;

    private ExecutionService executionService;

    private RepositoryService repositoryService;

    @Inject
    public JbpmFacadeImpl(TaskService taskService, ExecutionService executionService,
            RepositoryService repositoryService) {
        this.taskService = taskService;
        this.executionService = executionService;
        this.repositoryService = repositoryService;
    }

    /**
     * @see JbpmFacade#getIssueId(String)
     */
    public String getIssueId(String taskId) {
        String issueId = (String) taskService.getVariable(taskId, VAR_ISSUE_ID);
        return issueId;
    }

    /**
     * @see JbpmFacade#startIssueProcess(Issue)
     */
    public String startIssueProcess(Issue issue) {
        Map<String, Object> variables = new HashMap<String, Object>();
        // Voeg variabelen toe
        variables.put(VAR_ISSUE_ID, issue.getId());

        // Start process to handle form
        ProcessInstance processInstance = executionService.startProcessInstanceByKey("PROCESS_A", variables);

        // Zoek nu de taakId op van de eerste wachtende taak van dit proces.
        // to do controleer of de teruggegeven lijst op volgorde is, of in ieder
        // geval de laatste openstaande taak als laatste bevat
        List<Task> list = taskService.createTaskQuery().processInstanceId(processInstance.getId()).list();
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        Task task = list.get(list.size() - 1);
        return task.getId();
    }

    /**
     * @see JbpmFacade#getIssueIdsVoorTaak(String...)
     */
    public Map<String, String> getIssueIdsVoorTaak(String... taaknamen) {
        Map<String, String> issueIds = new HashMap<String, String>();
        for (String taaknaam : taaknamen) {
            List<Task> taskList = taskService.createTaskQuery().activityName(taaknaam).list();
            for (Task task : taskList) {
                String issueId = getIssueId(task.getId());
                if (!issueIds.containsKey(issueId)) {
                    issueIds.put(issueId, task.getId());
                }
            }
        }
        return issueIds;
    }

    /**
     * @see JbpmFacade#getProcessDefinitionKeys()
     */
    public Set<String> getProcessDefinitionKeys() {
        ProcessDefinitionQuery query = repositoryService.createProcessDefinitionQuery();
        List<ProcessDefinition> definitions = query.list();
        Set<String> keys = new TreeSet<String>();
        for (ProcessDefinition definition : definitions) {
            keys.add(definition.getKey());
        }
        return keys;
    }

    /**
     * @see JbpmFacade#isIssueInTaak(String, Issue)
     */
    public boolean isIssueInTaak(String taaknaam, Issue issue) {
        Task issueTask = null;
        List<Task> taskList = taskService.createTaskQuery().activityName(taaknaam).list();
        for (Task task : taskList) {
            if (issue.getId().equals(getIssueId(task.getId()))) {
                issueTask = task;
                break;
            }
        }

        if (issueTask == null) {
            return false;
        }

        return true;
    }
}
