package nu.helmers.jbpmtest.wiring.modules;

import nu.helmers.jbpmtest.wiring.startup.JbpmProcessDefinitionsLoader;

import org.hibernate.SessionFactory;
import org.jbpm.api.Configuration;
import org.jbpm.api.ExecutionService;
import org.jbpm.api.ProcessEngine;
import org.jbpm.api.RepositoryService;
import org.jbpm.api.TaskService;

import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import com.google.inject.Singleton;

/** Production module, that binds real jBPM implementations to the jBPM interfaces */
public class JbpmLayerModule extends AbstractModule {

    @Override
    protected void configure() {
        bind(JbpmProcessDefinitionsLoader.class).asEagerSingleton();
    }

    /**
     * Create a jBPM ProcessEngine, that uses *our* SessionFactory instance, instead of having jBPM load its own.
     * 
     * Note: Make sure that in a test environment (Junit), the jndi data source is available before the session factory
     * is requested. In other words, that {@link JndiDataSourceCreator} is executed before provideProcessEngine is
     * called. This should work fine, because JndiDataSourceCreator is loaded as eager singleton.
     * 
     * @param hibernateSessionFactory
     *            the session factory to wire into jBPM.
     */
    @Provides
    @Singleton
    ProcessEngine provideProcessEngine(SessionFactory hibernateSessionFactory) {
        // TODO Why is the SessionFactory, retrieved by jBPM later on, different from the one passed in here?
        // 
        // Put a break point here, and in jBPM's HibernateSessionDescriptor (line 59) 
        // and HibernatesessionFactoryDescriptor (line 66) and you'll see what I mean..
        // object Id is different, probably jBPM loads (again, because Guice HibernateConfigurationProvider 
        // already did) a new hibernate Configuration based on hibernate.cfg.xml,
        // and uses that. Problem: This jBPM hibernate configuration will not have the properties 
        // (from jbpmtest-dao) that the Guice version does have (see HibernateConfigurationProvider)!
        return new Configuration().setResource("jbpm/jbpm.cfg.xml").setHibernateSessionFactory(hibernateSessionFactory)
                .buildProcessEngine();
    }

    @Provides
    ExecutionService provideExecutionService(ProcessEngine processEngine) {
        return processEngine.getExecutionService();
    }

    @Provides
    RepositoryService provideRepositoryService(ProcessEngine processEngine) {
        return processEngine.getRepositoryService();
    }

    @Provides
    TaskService provideTaskService(ProcessEngine processEngine) {
        return processEngine.getTaskService();
    }
}
