package nu.helmers.jbpmtest.wiring.modules;

import nu.helmers.jbpmtest.logging.LogInterceptor;
import nu.helmers.jbpmtest.logging.LogMe;
import nu.helmers.jbpmtest.service.JbpmFacade;
import nu.helmers.jbpmtest.service.IIssueService;
import nu.helmers.jbpmtest.service.impl.JbpmFacadeImpl;
import nu.helmers.jbpmtest.service.impl.IssueServiceImpl;

import org.slf4j.LoggerFactory;

import com.google.inject.AbstractModule;
import com.google.inject.matcher.Matchers;

/**
 * Module that binds services. In addition it binds a logging interceptor to all annotated service methods.
 */
public class ServiceLayerModule extends AbstractModule {

    protected void configure() {
        install(new JbpmLayerModule());
        bindServices();
    }

    private void bindServices() {
        bindService(IIssueService.class, IssueServiceImpl.class);
        bindService(JbpmFacade.class, JbpmFacadeImpl.class);
    }

    private <T> void bindService(Class<T> iface, Class<? extends T> impl) {
        bind(iface).to(impl);

        bindInterceptor(Matchers.subclassesOf(impl), Matchers.annotatedWith(LogMe.class), new LogInterceptor(
                LoggerFactory.getLogger(impl)));
    }
}
