package nu.helmers.jbpmtest.service.impl;

import static org.junit.Assert.assertEquals;

import java.util.Arrays;

import nu.helmers.commons.guice.test.MockInjector;
import nu.helmers.commons.guice.test.MockModule;
import nu.helmers.jbpmtest.dao.issue.IIssueDao;
import nu.helmers.jbpmtest.domain.issue.Issue;
import nu.helmers.jbpmtest.fixtures.FixturesIssue;
import nu.helmers.jbpmtest.service.JbpmFacade;
import nu.helmers.jbpmtest.wiring.modules.MockedDataLayerModule;
import nu.helmers.jbpmtest.wiring.modules.MockedServiceLayerModule;

import org.easymock.EasyMock;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class IssueServiceTest {

    private IssueServiceImpl issueService;

    private IIssueDao mockIssueDao;

    private MockInjector injector;

    private Issue issue;

    private JbpmFacade mockTaakService;

    @Before
    public void setup() {
        injector = new MockInjector(Arrays.asList(new MockModule[] { new MockedDataLayerModule(),
                new MockedServiceLayerModule() }));

        issueService = injector.get(IssueServiceImpl.class); // not a mock!
        mockIssueDao = injector.get(IIssueDao.class);
        mockTaakService = injector.get(JbpmFacade.class);

        issue = FixturesIssue.makeNewGbaIssue();
    }

    @After
    public void teardown() {
        mockIssueDao = null;
        mockTaakService = null;
        issueService = null;
    }

    @Test
    public void testCreate() {
        String expectedTaakId = "taakId";

        mockIssueDao.persist(issue);
        EasyMock.expect(mockTaakService.startIssueProcess(issue)).andReturn(expectedTaakId);

        injector.replay();
        String taakId = issueService.create(issue);
        injector.verify();

        assertEquals(expectedTaakId, taakId);
    }

    @Test
    public void testGet() {
        final Long id = 1L;
        EasyMock.expect(mockIssueDao.get(id)).andReturn(issue);

        injector.replay();
        Issue returnedIssue = issueService.get(id);
        injector.verify();

        assertEquals(issue, returnedIssue);
    }

    @Test
    public void testUpdate() {
        mockIssueDao.update(issue);

        injector.replay();
        issueService.update(issue);
        injector.verify();
    }
}
