package nu.helmers.jbpmtest.wiring.modules;

import javax.naming.NamingException;

import org.apache.commons.dbcp.BasicDataSource;
import org.springframework.mock.jndi.SimpleNamingContextBuilder;

import com.google.inject.Inject;
import com.google.inject.name.Named;

/** Creates a data source, and stores it under the jndi name that jbpmtest-dao uses. */
class JndiDataSourceCreator {

    /**
     * The jndi name under which jbpmtest-dao looks up data source (property hibernate.connection.datasource)
     */
    public static final String JNDI_NAME = "java:/comp/env/jdbc/jbpmtestDS";

    /**
     * File containing properties needed to construct a data source. Only for test purposes, because in a production
     * environment the app server creates data sources.
     */
    public final static String HSQL_PROPERTIES = "/JndiDataSourceCreator-hsql.properties";

    public final static String JDBC_DRIVER = "jbpmtest.jdbc.driver";

    public final static String JDBC_URL = "jbpmtest.jdbc.url";

    public final static String JDBC_USER = "jbpmtest.jdbc.user";

    public final static String JDBC_PASSWORD = "jbpmtest.jdbc.password";

    @Inject
    public JndiDataSourceCreator(@Named(JDBC_DRIVER) String jdbcDriver, @Named(JDBC_URL) String jdbcUrl,
            @Named(JDBC_USER) String jdbcUser, @Named(JDBC_PASSWORD) String jdbcPassword) {

        /*
         * If you don't load the jdbc driver here, you'll get something like this...:
         * 
         * Caused by: org.apache.commons.dbcp.SQLNestedException: Cannot create JDBC driver of class '' for connect URL
         * 'jdbc:hsqldb:mem:.'
         * 
         * Caused by: java.sql.SQLException: No suitable driver
         */
        try {
            Class.forName(jdbcDriver);
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }

        BasicDataSource dataSource = new BasicDataSource();
        dataSource.setUrl(jdbcUrl);
        dataSource.setUsername(jdbcUser);
        dataSource.setPassword(jdbcPassword);

        SimpleNamingContextBuilder builder = new SimpleNamingContextBuilder();
        builder.bind(JNDI_NAME, dataSource);
        try {
            builder.activate();
        } catch (NamingException e) {
            throw new RuntimeException(e);
        }
    }
}
