package nu.helmers.jbpmtest.wiring.modules;

import nu.helmers.commons.guice.PropertiesModule;

import org.springframework.jdbc.core.JdbcTemplate;

import com.google.inject.AbstractModule;
import com.google.inject.Module;

/**
 * This module creates a data source for an in-memory hsql database, and makes it available under the jndi name that
 * jbpmtest-dao will look it up under.
 * <p/>
 * In addition it configures a JdbcTemplateProvider, so that you can have Guice inject a JdbcTemplate in your test code
 * where needed (to do direct jdbc, for instance in database test utilities).
 * <p/>
 * There's no production equivalent of this module; in a container (Jetty, OC4J, ..) the container makes the jndi data
 * source.
 */
public class TestDataSourceModule extends AbstractModule implements Module {

    @Override
    protected void configure() {
        install(new PropertiesModule(JndiDataSourceCreator.HSQL_PROPERTIES)); // doe maar hsql..
        bind(JndiDataSourceCreator.class).asEagerSingleton(); // make a jndi datasource
        bind(JdbcTemplate.class).toProvider(new JdbcTemplateProvider()); // uses jndi datasource
    }

}
